#![no_main]
#![no_std]
#[allow(unused_extern_crates)]
extern crate panic_halt;

use embedded_hal::digital::v2::{OutputPin, ToggleableOutputPin};

use lpc43xx_hal::{
    timer::{self, EMC0_A, MR0I_A, MR0R_A, MR0S_A, NoMatchPin, NoCapturePin},
    io::pin::{split_ports, Gpio, IntoGpioPin, Output, Pin, PinSplit, U7},
    pac,
};
use rtfm::app;

static BLINK_PERIOD: u32 = 16384;

type Test0Timer = timer::Timer<pac::TIMER0, NoMatchPin, NoMatchPin, NoMatchPin, NoMatchPin, NoCapturePin, NoCapturePin, NoCapturePin, NoCapturePin>;

#[app(device = lpc43xx_pac)]
const APP: () = {
    static mut TIMER: Test0Timer = ();
    static mut LED: Pin<U7, U7, Gpio<Output>> = ();

    #[init]
    fn init() -> init::LateResources {
        // Configure timer
        let mut t: Test0Timer = Test0Timer::new_no_pins(device.TIMER0);
        t.start_timer(false);
        t.set_match(0, BLINK_PERIOD);
        t.set_prescaler(1024, Some(0));
        // Safe since the timer is stopped
        unsafe {
            t.mr0(MR0I_A::INTERRUPT, MR0R_A::RESET, MR0S_A::NO_STOP, EMC0_A::NOP);
        }
        t.start_timer(true);

        // Configure GPIO
        // TODO feature-gated pin configuration
        let mut led_pin = split_ports(device.SCU, device.GPIO_PORT)
            .port_7
            .split_pins()
            .p7_7
            .into_gpio()
            .into_output();

        led_pin.set_low().unwrap();
        init::LateResources {
            TIMER: t,
            LED: led_pin,
        }
    }

    #[idle]
    fn idle() -> ! {
        loop {}
    }

    #[interrupt(resources = [TIMER, LED])]
    fn TIMER0() {
        resources.LED.toggle().unwrap();
        resources
            .TIMER
            .clear_interrupt(timer::InterruptType::MatchChannel, 0);
    }
};
