#![no_main]
#![no_std]

extern crate panic_halt;

use cortex_m::asm; 

use rtfm::app;

use lpc43xx_hal as driver;
use driver::eeprom;
use driver::eeprom::{Read, Write};
use driver::time::U32Ext;

#[app(device = lpc43xx_pac)]
const APP: () = {
    static mut EEPROM: eeprom::EEPROM = ();

    #[init]
    fn init() -> init::LateResources {

        let eeprom_reg = device.EEPROM;

        let eeprom = eeprom::EEPROM::new(eeprom_reg, 12.mhz());

        init::LateResources {
            EEPROM: eeprom,
        }
    }

    #[idle(resources = [EEPROM])]
    fn idle() -> ! {

        let eeprom = resources.EEPROM;

        let write_data: [u32; 8] = [1,2,3,4,5,6,7,8];
        let mut read_buffer: [u32; 8] = [0; 8];

        match eeprom.write(29, &write_data) {
            Ok(_) => (),
            Err(_) => loop { cortex_m::asm::nop() },
        };

        match eeprom.read(29, &mut read_buffer) {
            Ok(_) => (),
            Err(_) => loop { cortex_m::asm::nop() },
        };

        asm::bkpt();

        assert!(
            write_data == read_buffer,
            "Data should be the same");


        loop {
            cortex_m::asm::wfi();
        }
    }
};
