#![no_main]
#![no_std]
#[allow(unused_extern_crates)]
extern crate panic_halt;

use embedded_hal::digital::v2::{OutputPin, ToggleableOutputPin};

#[allow(unused)]
use lpc43xx_hal::io::pin::{U7, U1, U16, U15, U14, U18};
use lpc43xx_hal::{
    timer::{EMC0_A, EMC3_A, MR0I_A, MR3I_A, MR0R_A, MR3R_A, MR0S_A, MR3S_A, NoMatchPin, NoCapturePin},
    io::pin::{split_ports, Gpio, IntoGpioPin, Output, Pin, PinSplit, Uninit},
    pac, timer,
};
use rtfm::app;

const PERIOD: u32 = 0x1000;

type Test0Match = Pin<U1, U16, Uninit>;
//type Test1Match = Pin<U1, U15, Uninit>;
//type Test2Match = Pin<U1, U14, Uninit>;
//type Test3Match = Pin<U1, U18, Uninit>;
type Test0Timer = timer::Timer<pac::TIMER0, Test0Match, NoMatchPin, NoMatchPin, NoMatchPin, NoCapturePin, NoCapturePin, NoCapturePin, NoCapturePin>;

#[app(device = lpc43xx_pac)]
const APP: () = {
    static mut TIMER: Test0Timer = ();
    static mut LED: Pin<U7, U7, Gpio<Output>> = ();

    #[init]
    fn init() -> init::LateResources {
        let ports = split_ports(device.SCU, device.GPIO_PORT);
        let pwm_pin = ports.port_1.split_pins().p1_16;
        // Configure timer
        let mut t: Test0Timer = Test0Timer::new(device.TIMER0, Some(pwm_pin), None, None, None, None, None, None, None);
        t.start_timer(false);
        // PWM high time
        t.set_match(0, PERIOD/3);
        // PWM period in timer clocks
        t.set_match(3, PERIOD);
        t.set_prescaler(0x1000, Some(0));
        // Safe, because the timer is stopped. There can be no concurrent access to `emr`.
        unsafe {
            // just clear EMR on match
            t.mr0(MR0I_A::NO_INTERRUPT, MR0R_A::NO_RESET, MR0S_A::NO_STOP, EMC0_A::CLEAR);
            // interrupt and reset the timer
            t.mr3(MR3I_A::INTERRUPT, MR3R_A::RESET, MR3S_A::NO_STOP, EMC3_A::NOP);
        }
        t.start_timer(true);

        // Configure GPIO
        let mut led_pin = ports
            .port_7
            .split_pins()
            .p7_7
            .into_gpio()
            .into_output();

        led_pin.set_low().unwrap();
        init::LateResources {
            TIMER: t,
            LED: led_pin,
        }
    }

    #[interrupt(resources = [TIMER, LED])]
    fn TIMER0() {
        resources
            .TIMER
            .clear_interrupt(timer::InterruptType::MatchChannel, 3);

        // Safety: The read-modify-write of `emr` can happen concurrently with the peripheral
        // clearing the external match register if some other task blocks the TIMER0 interrupt from
        // match event 0 until match event 3. This is especially likely to happen when the duty
        // cycle is very close to 0%. This will produce close to 100% duty cycle for a single PWM
        // period.
        //
        // In this case, the duty cycle is not close to 0% and there are no tasks that can block
        // this one.
        unsafe {
            resources.TIMER.write_em0(true);
        }
        resources.LED.toggle().unwrap();
    }
};
