//! Using a device crate
//!
//! Crates generated using [`svd2rust`] are referred to as device crates. These crates provide an
//! API to access the peripherals of a device.
//!
//! [`svd2rust`]: https://crates.io/crates/svd2rust
//!
//! This example depends on the [`stm32f103xx`] crate so you'll have to add it to your Cargo.toml.
//!
//! [`stm32f103xx`]: https://crates.io/crates/stm32f103xx
//!
//! ```
//! $ edit Cargo.toml && tail $_
//! [dependencies.stm32f103xx]
//! features = ["rt"]
//! version = "0.10.0"
//! ```
//!
//! ---
#![no_main]
#![no_std]

#[allow(unused_extern_crates)]
extern crate panic_halt;

use cortex_m_semihosting::hprint;
use lpc43xx_hal::pac::Interrupt;

use rtfm::app;

#[app(device = lpc43xx_pac)]
const APP: () = {
    #[init]
    fn init() {
        rtfm::pend(Interrupt::UART1);
        hprint!("Initializing...").unwrap();
    }

    #[idle]
    fn idle() -> ! {
        rtfm::pend(Interrupt::UART1);
        loop {}
    }

    #[interrupt]
    fn UART1() {
        static mut TIMES: u32 = 0;
        *TIMES += 1;
        hprint!(".").unwrap();
    }
};
