#![no_main]
#![no_std]
#[allow(unused_extern_crates)]
extern crate panic_halt;

static STRING: [u8; 13] = [
    0x48, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64, 0x21, 0x20,
];
static TIMER_PERIOD: u32 = 16384;

use core::fmt::Write as OtherWrite;
use rtfm::app;

// UART driver
use lpc43xx_hal::{
    clock::uart_pll,
    io::pin::{split_ports, PinSplit},
    pac,
    serial::{Read, Serial},
    timer::{Timer, EMC0_A, InterruptType, MR0I_A, MR0R_A, MR0S_A, NoMatchPin, NoCapturePin},
};
use pac::usart0::lcr::{BC_A, DLAB_A, PE_A, PS_A, SBS_A, WLS_A};

type Timer0 = Timer<pac::TIMER0, NoMatchPin, NoMatchPin, NoMatchPin, NoMatchPin, NoCapturePin, NoCapturePin, NoCapturePin, NoCapturePin>;

#[app(device = lpc43xx_pac)]
const APP: () = {
    static mut UART: Serial<pac::USART3> = ();
    static mut TIMER: Timer0 = ();
    static mut IDX: u8 = ();

    #[init]
    fn init() -> init::LateResources {
        let idx: u8 = 0;

        // Pins
        let port_9 = split_ports(device.SCU, device.GPIO_PORT)
            .port_9
            .split_pins();
        let uart_tx = port_9.p9_3;
        let uart_rx = port_9.p9_4;

        let mut serial =
            Serial::<pac::USART3>::new(device.USART3, uart_tx, uart_rx, 0x41, 0x0, 0x0);
        serial.lcr(
            WLS_A::_8_BIT_CHARACTER_LENGTH,
            SBS_A::_1_STOP_BIT,
            PE_A::DISABLE_PARITY_GENERERATION,
            PS_A::ODD_PARITY,
            BC_A::DISABLED,
            DLAB_A::DISABLED,
        );
        serial.ier(true, false, false, false, false);

        // Configure timer
        let mut timer = Timer0::new_no_pins(device.TIMER0);
        timer.start_timer(false);
        timer.set_match(0, TIMER_PERIOD);
        timer.set_prescaler(1024, Some(0));
        // Safe since the timer is stopped
        unsafe {
            timer.mr0(MR0I_A::INTERRUPT, MR0R_A::RESET, MR0S_A::NO_STOP, EMC0_A::NOP);
        }
        timer.start_timer(true);

        unsafe {
            uart_pll(&mut device.CGU);
        }

        init::LateResources {
            UART: serial,
            TIMER: timer,
            IDX: idx,
        }
    }

    #[idle]
    fn idle() -> ! {
        loop {}
    }

    #[interrupt(resources = [IDX, TIMER, UART])]
    fn TIMER0() {
        *resources.IDX = (*resources.IDX + 1) % 13;
        write!(resources.UART, "{}", STRING[*resources.IDX as usize] as char).unwrap();
        resources
            .TIMER
            .clear_interrupt(InterruptType::MatchChannel, 0);
    }

    #[interrupt(resources = [UART])]
    fn USART3() {
        let iir = resources.UART.read_iir();
        let val = resources.UART.read();
        write!(resources.UART, "\r\nIIR: {:?}\r\n", iir).unwrap();
        val.unwrap();
    }
};
