mod uart_pll;
pub use uart_pll::uart_pll;
use crate::pac;

/// Initialize PLL
pub unsafe fn setup_pll1(cgu: &pac::CGU, msel: u8, nsel: u8, psel: u8) {
    // Select the IRC as BASE_M4_CLK source
    cgu.base_m4_clk.write(|w| w.clk_sel().bits(0x1));

    // Enable the crystal oscillator
    cgu.xtal_osc_ctrl.modify(|_, w| w.hf().bit(false));
    cgu.xtal_osc_ctrl.modify(|_, w| w.enable().bit(false));

    // Wait 250us
    for i in 0..3_000 {
        core::ptr::read_volatile(&i);
    }

    cgu.pll1_ctrl.write(|w| {
        w.pd()
            .bit(false)
            .clk_sel()
            .bits(0x6) // Select the crystal oscillator as clock source
            .msel()
            .bits(msel) //16 => M=17
            .nsel()
            .bits(nsel) // 0 => N=1
            .psel()
            .bits(psel) // 0 => P=1
            .fbsel()
            .bit(true) //true => output feedback
            .bypass()
            .bit(false) // 0=> normal PLL
            .direct()
            .bit(true)// 0=> use post-divider
            //=> no post divider => direct mode
            // Clk = M * COSC/N
            // CCO = CLK = 204 MHz
            //156MHz < CCO < 320 MHz => OK 
    });

    // Wait for the PLL to lock
    while !cgu.pll1_stat.read().lock().bit() {
        cortex_m::asm::nop();
    }


    // Select PLL1 as base_m4_clk source
    cgu.base_m4_clk.modify(|_, w| w.clk_sel().bits(0x09));

    // Wait for 50us
    for i in 0..600 {
        core::ptr::read_volatile(&i);
    }

    // NOTE: Don't configure any clock outputs here
}
