use crate::pac;
use pac::creg::creg0::{ALARMCTRL_A, SAMPLECTRL_A, USB0PHY_A, BODLVL1_A, BODLVL2_A, EN1KHZ_A, EN32KHZ_A, PD32KHZ_A, RESET32KHZ_A, WAKEUP0CTRL_A, WAKEUP1CTRL_A}; 
pub fn identify(creg: &pac::CREG) -> u32 {
    creg.chipid.read().id().bits()
}

pub fn control_1khz(creg: &pac::CREG, en: EN1KHZ_A) {
    creg.creg0.modify(|_, w| w.en1khz().variant(en));
}

pub fn control_32khz(creg: &pac::CREG, en: EN32KHZ_A, rst: RESET32KHZ_A, pd: PD32KHZ_A) {
    creg.creg0.modify(|_, w| {
        w.en32khz()
            .variant(en)
            .reset32khz()
            .variant(rst)
            .pd32khz()
            .variant(pd)
    });
}

pub fn control_usb(creg: &pac::CREG, pd: USB0PHY_A) {
    creg.creg0.modify(|_, w| w.usb0phy().variant(pd));
}

pub fn control_alarm(creg: &pac::CREG, status: ALARMCTRL_A) {
    creg.creg0.modify(|_, w| w.alarmctrl().variant(status));
}

pub fn control_bod(creg: &pac::CREG, int: BODLVL1_A, rst: BODLVL2_A) {
    creg.creg0
        .modify(|_, w| w.bodlvl1().variant(int).bodlvl2().variant(rst));
}

pub fn control_sample(creg: &pac::CREG, route: SAMPLECTRL_A) {
    // TODO: fix cut names
    // TODO: can we do anything about the ambiguity=
    creg.creg0.modify(|_, w| w.samplectrl().variant(route));
}

pub fn control_wakeup(creg: &pac::CREG, wu0: WAKEUP0CTRL_A, wu1: WAKEUP1CTRL_A) {
    creg.creg0
        .modify(|_, w| w.wakeup0ctrl().variant(wu0).wakeup1ctrl().variant(wu1));
}
