use crate::pac;

pub use pac::creg::creg6::{
    CTOUTCTRL_A, EMC_CLK_SEL_A, ETHMODE_A, I2S0_RX_SCK_IN_SEL_A, I2S0_TX_SCK_IN_SEL_A,
    I2S1_RX_SCK_IN_SEL_A, I2S1_TX_SCK_IN_SEL_A,
};

/// Set the ethernet mode
pub fn control_eth(creg: &pac::CREG, mode: ETHMODE_A) {
    creg.creg6.modify(|_, w| w.ethmode().variant(mode));
}

/// Select whether or not to OR the SCT and timer outputs
pub fn control_ctout(creg: &pac::CREG, combine: CTOUTCTRL_A) {
    creg.creg6.modify(|_, w| w.ctoutctrl().variant(combine));
}

/// Set I²S0 input
pub fn control_i2s0(creg: &pac::CREG, rxin: I2S0_RX_SCK_IN_SEL_A, txin: I2S0_TX_SCK_IN_SEL_A) {
    creg.creg6.modify(|_, w| {
        w.i2s0_rx_sck_in_sel()
            .variant(rxin)
            .i2s0_tx_sck_in_sel()
            .variant(txin)
    });
}

/// Set I²S1 input
pub fn control_i2s1(creg: &pac::CREG, rxin: I2S1_RX_SCK_IN_SEL_A, txin: I2S1_TX_SCK_IN_SEL_A) {
    creg.creg6.modify(|_, w| {
        w.i2s1_rx_sck_in_sel()
            .variant(rxin)
            .i2s1_tx_sck_in_sel()
            .variant(txin)
    });
}

/// Configure EMC clock division
pub fn control_emcclk(creg: &pac::CREG, clksel: EMC_CLK_SEL_A) {
    creg.creg6.modify(|_, w| w.emc_clk_sel().variant(clksel));
}
