//! Provides debug printing over UART
pub use core::panic::PanicInfo;

#[macro_export]
macro_rules! uart_panic_handler {
    ($UART: path) => {
        use core::fmt::Write as DebugWrite;
        use $crate::heapless::String;
        static mut PANIC_UART: Option<Serial<$UART>> = None;

        #[panic_handler]
        fn panic(info: &PanicInfo) -> ! {
            unsafe {
                match &mut PANIC_UART {
                    None => loop {},
                    Some(uart) => {
                        uart.reset_tx_fifo();
                        write!(uart, "panic: {:?}\r\n",
                            *info.message().unwrap_or(
                                &format_args!("could not unpack panic message")
                            )).unwrap_or(());
                    }
                }
            }

            cortex_m::asm::bkpt();
            loop {
                cortex_m::asm::nop();
            }
        }
    };
}
