use crate::pac;
use core::marker::PhantomData;
pub struct U0;
pub struct U1;
pub struct U2;
pub struct U3;
pub struct U4;
pub struct U5;
pub struct U6;
pub struct U7;
pub struct U8;
pub struct U9;
pub struct U10;
pub struct U11;
pub struct U12;
pub struct U13;
pub struct U14;
pub struct U15;
pub struct U16;
pub struct U17;
pub struct U18;
pub struct U19;
pub struct U20;
pub struct A;
pub struct B;
pub struct C;
pub struct D;
pub struct E;
pub struct F;

/// Extension trait to split the pin configuration peripherals in independent pins and registers
pub trait PinSplit {
    /// The parts to split the GPIO into
    type Parts;

    /// Splits the pin block into independent pins and registers
    fn split_pins(self) -> Self::Parts;
}

//pub struct Pin<MODE> {
//    _mode: PhantomData<MODE>,
//}

pub struct Pin<X, N, MODE> {
    _port: PhantomData<X>,
    _pin: PhantomData<N>,
    _mode: PhantomData<MODE>,
}

impl<X, N, MODE> Pin<X, N, MODE> {
    /// Materializes a `Pin` out of thin air. For internal use.
    ///
    /// # Safety
    /// Owning a `Pin` implies exclusive access to the register fields that configure that pin.
    pub unsafe fn new() -> Self {
        Pin {
            _port: PhantomData,
            _pin: PhantomData,
            _mode: PhantomData,
        }
    }
}

// type states
pub struct Gpio<MODE> {
    _mode: PhantomData<MODE>,
}
//pub struct Input<MODE> { _mode: PhantomData<MODE> }
pub struct Input;
//pub struct Floating;
//pub struct PullDown;
//pub struct PullUp;
pub struct OpenDrain;

//pub struct Output { _mode: PhantomData<MODE> }
pub struct Output;
pub struct Uninit;
//pub struct PushPull;
//pub struct Analog;
//pub struct Speed { ... }

pub trait SlaveSelect {
    fn select(&mut self);
    fn unselect(&mut self);
    fn is_selected(&self) -> bool;
}

pub struct NoSlaveSelect {}
impl SlaveSelect for NoSlaveSelect {
    fn select(&mut self) {}
    fn unselect(&mut self) {}
    fn is_selected(&self) -> bool {
        false
    }
}

pub trait OpenDrainPin {
    type Error;
    fn set_low(&mut self) -> Result<(), Self::Error>;
    fn set_floating(&mut self) -> Result<(), Self::Error>;

}

pub enum TriggerMode {
    Disabled,
    EdgeFalling,
    EdgeRising,
    EdgeBoth,
    // TODO Level low/high
}

pub trait IntoGpioPin {
    type X;
    type N;
    fn into_gpio(self) -> Pin<Self::X, Self::N, Gpio<Uninit>>;
    fn get_gpio_location(&self) -> (u8, u8);
}

pub struct GpioPinInt<I> {
    _int_number: PhantomData<I>,
}

macro_rules! gpio_interrupts {
    ($($pintselx:ident: [$($i: literal),+]),+) => {paste::item!{
        use crate::pac::{GPIO_PIN_INT, SCU};

        pub struct GpioInterrupts {
            $($(pub [<pin_int_ $i>]: GpioPinInt<[<U $i>]>,)+)+
        }
        impl GpioInterrupts {
            pub fn new(_pin_int: pac::GPIO_PIN_INT) -> Self {
                GpioInterrupts {
                    $($([<pin_int_ $i>]: GpioPinInt { _int_number: PhantomData },)+)+
                }
            }
        }

        $($(impl GpioPinInt<[<U $i>]> {
            /// Selects the pin to receive interrupts from. Does not configure the pin to be
            /// GPIO.
            pub fn set_interrupt_source<P: IntoGpioPin>(&mut self, pin: &P) {
                let (gpio_bank, gpio_pin) = pin.get_gpio_location();
                let scu = unsafe { &(*SCU::ptr()) };
                scu.$pintselx.modify(|_, w| unsafe { w.
                    [<intpin $i>]()
                    .bits(gpio_pin)
                    .[<portsel $i>]()
                    .bits(gpio_bank)
                });
            }

            pub fn trigger_mode(&mut self, mode: TriggerMode) {
                let pin_int = unsafe { &(*GPIO_PIN_INT::ptr()) };
                match mode {
                    TriggerMode::Disabled => {
                        pin_int.cienr.write(|w| w.[<cenrl $i>]().set_bit());
                        pin_int.cienf.write(|w| w.[<cenaf $i>]().set_bit());
                    }
                    TriggerMode::EdgeFalling => {
                        pin_int.isel.write(|w| w.[<pmode $i>]().clear_bit());
                        pin_int.cienr.write(|w| w.[<cenrl $i>]().set_bit());
                        pin_int.sienf.write(|w| w.[<setenaf $i>]().set_bit());
                    }
                    TriggerMode::EdgeRising => {
                        pin_int.isel.write(|w| w.[<pmode $i>]().clear_bit());
                        pin_int.sienr.write(|w| w.[<setenrl $i>]().set_bit());
                        pin_int.cienf.write(|w| w.[<cenaf $i>]().set_bit());
                    }
                    TriggerMode::EdgeBoth => {
                        pin_int.isel.write(|w| w.[<pmode $i>]().clear_bit());
                        pin_int.sienr.write(|w| w.[<setenrl $i>]().set_bit());
                        pin_int.sienf.write(|w| w.[<setenaf $i>]().set_bit());
                    }
                }
            }

            pub fn disable(&mut self) {
                self.trigger_mode(TriggerMode::Disabled);
            }

            pub fn clear_pending_bit(&mut self) {
                let pin_int = unsafe { &(*GPIO_PIN_INT::ptr()) };
                pin_int.ist.write(|w| w.[<pstat $i>]().set_bit());
            }
        })+)+
    }}
}

//gpio_interrupts![0,1,2,3,4,5,6,7];
gpio_interrupts! {pintsel0: [0,1,2,3], pintsel1: [4,5,6,7]}

macro_rules! ports {
    ($($x:tt ($X:ty): [
      $($n:tt ($N:ty): [
        gpio[$gpio_bank:literal][$gpio_pin:literal]: ($sfs_mode:ident, $default_mode:ty),
      ], )+
    ],)+) => {paste::item!{$(
        pub struct [<Port $X>] {
            // prevent users from constructing us, because then pins could be aliased
            _marker: PhantomData<()>,
        }

        pub mod [<port_ $x>] {
            use super::{[<Port $X>], PinSplit, Pin, Gpio, Input, Output, OpenDrain, OpenDrainPin, Uninit, SlaveSelect, IntoGpioPin};
            use core::marker::PhantomData;
            use embedded_hal::digital::v2::{
                InputPin, OutputPin, StatefulOutputPin, ToggleableOutputPin
            };
            use crate::pac::{GPIO_PORT, SCU};
            #[allow(unused_imports)]
            use super::{
                U0, U1, U2, U3, U4, U5, U6, U7, U8, U9,
				U10, U11, U12, U13, U14, U15, U16, U17, U18, U19, U20,
                A, B, C, D, E, F,
            };


            pub struct Parts {
                $(
                    /// Pin
                    pub [<p $x _ $n>]: Pin<$X, $N, $default_mode>,
                 )+
            }

            impl PinSplit for [<Port $X>] {
                type Parts = Parts;
                fn split_pins(self) -> Self::Parts {
                    Parts {
                        $([<p $x _ $n>]: Pin {
                            _port: PhantomData, _pin: PhantomData, _mode: PhantomData
                        },)+
                    }
                }
            }

            $(
                impl<MODE> IntoGpioPin for Pin<$X, $N, MODE> {
                    type X = $X;
                    type N = $N;

                    fn into_gpio(self) -> Pin<$X, $N, Gpio<Uninit>> {
                        // TODO set the ADC pin configuration as well, if this pin has such
                        // configuration
                        unsafe{
                            (*SCU::ptr()).[<sfsp $x _ $n>].write(|w| {
                                w.ezi().enable_input_buffer().mode().$sfs_mode()
                            });
                        }
                        Pin { _port: PhantomData, _pin: PhantomData, _mode: PhantomData }
                    }

                    fn get_gpio_location(&self) -> (u8, u8) {
                        ($gpio_bank, $gpio_pin)
                    }
                }

                impl<MODE> Pin<$X, $N, Gpio<MODE>> {
                    pub fn into_input(self) -> Pin<$X, $N, Gpio<Input>> {
                        unsafe {
                            // reset the SCU register
                            (*SCU::ptr()).[<sfsp $x _ $n>].write(|w| {
                                w.ezi().enable_input_buffer().mode().$sfs_mode()
                            });
                            // direction: input
                            (*GPIO_PORT::ptr()).dir[$gpio_bank].modify(|_, w| w.[<dirp $gpio_pin>]().clear_bit());
                            Pin { _port: PhantomData, _pin: PhantomData, _mode: PhantomData }
                        }
                    }

                    pub fn into_output(self) -> Pin<$X, $N, Gpio<Output>> {
                        unsafe {
                            // reset the SCU register
                            //(*SCU::ptr()).[<sfsp $x _ $n>].write(|w| w);
                            (*SCU::ptr()).[<sfsp $x _ $n>].write(|w| w.mode().$sfs_mode());
                            // direction: output
                            (*GPIO_PORT::ptr()).dir[$gpio_bank].modify(|_, w| w.[<dirp $gpio_pin>]().set_bit());
                            Pin { _port: PhantomData, _pin: PhantomData, _mode: PhantomData }
                        }
                    }

                    pub fn into_opendrain(self) -> Pin<$X, $N, Gpio<OpenDrain>> {
                        unsafe {
                            // reset the SCU register
                            (*SCU::ptr()).[<sfsp $x _ $n>].write(|w| w.mode().$sfs_mode());
                            // direction: output
                            (*GPIO_PORT::ptr()).dir[$gpio_bank].modify(|_, w| w.[<dirp $gpio_pin>]().set_bit());
                            Pin { _port: PhantomData, _pin: PhantomData, _mode: PhantomData }
                        }
                    }
                }

                impl InputPin for Pin<$X, $N, Gpio<Input>> {
                    type Error = ();
                    fn is_high(&self) -> Result<bool, ()> {
                        unsafe{
                            Ok((*GPIO_PORT::ptr()).pin[$gpio_bank].read().[<port $gpio_pin>]().bit_is_set())
                        }
                    }

                    fn is_low(&self) -> Result<bool, ()> {
                        self.is_high().map(|is_high| !is_high)
                    }
                }

                impl OutputPin for Pin<$X, $N, Gpio<Output>> {
                    type Error = ();
                    fn set_high(&mut self) -> Result<(), ()> {
                        unsafe{
                            (*GPIO_PORT::ptr()).set[$gpio_bank].write(|w| w.[<setp $gpio_pin>]().set_bit());
                        }
                        Ok(())
                    }

                    fn set_low(&mut self) -> Result<(), ()> {
                        unsafe{
                            (*GPIO_PORT::ptr()).clr[$gpio_bank].write(|w| w.[<clrp0 $gpio_pin>]().set_bit());
                        }
                        Ok(())
                    }
                }

                impl OpenDrainPin for Pin<$X, $N, Gpio<OpenDrain>> {
                    type Error = ();
                    fn set_floating(&mut self) -> Result<(), ()> {
                        unsafe{
                            (*GPIO_PORT::ptr()).dir[$gpio_bank].modify(|_, w| w.[<dirp $gpio_pin>]().clear_bit()); // High impedance input
                        }
                        Ok(())
                    }

                    fn set_low(&mut self) -> Result<(), ()> {
                        unsafe{
                            (*GPIO_PORT::ptr()).dir[$gpio_bank].modify(|_, w| w.[<dirp $gpio_pin>]().set_bit()); //Output
                            (*GPIO_PORT::ptr()).clr[$gpio_bank].write(|w| w.[<clrp0 $gpio_pin>]().set_bit()); //low
                        }
                        Ok(())
                    }
                }

                impl StatefulOutputPin for Pin<$X, $N, Gpio<Output>> {
                    fn is_set_high(&self) -> Result<bool, ()> {
                        unsafe {
                            Ok((*GPIO_PORT::ptr()).set[$gpio_bank].read().[<setp $gpio_pin>]().bit())
                        }
                    }

                    fn is_set_low(&self) ->  Result<bool, ()> {
                        Ok(!self.is_set_high().unwrap())
                    }
                }

                impl ToggleableOutputPin for Pin<$X, $N, Gpio<Output>> {
                    type Error = ();
                    fn toggle(&mut self) -> Result<(), ()> {
                        unsafe {
                            (*GPIO_PORT::ptr()).not[$gpio_bank].write(|w| w.[<notp $gpio_pin>]().set_bit());
                        }
                        Ok(())
                    }
                }


                // TODO Should be Error = !
                impl SlaveSelect for Pin<$X, $N, Gpio<Output>> {
                    fn select(&mut self) {
                        self.set_low().unwrap();
                    }
                    fn unselect(&mut self) {
                        self.set_high().unwrap();
                    }

                    fn is_selected(&self) -> bool {
                        self.is_set_low().unwrap()
                    }
                }
            )+
        })+

        pub struct Ports {
            pub sfsi2c0: pac::scu::SFSI2C0,
            $(pub [<port_ $x>]: [<Port $X>],)+
        }

        pub fn split_ports(_scu: pac::SCU, _gpio: pac::GPIO_PORT) -> Ports {
            Ports {
                // `ptr::read` is used because the deref stuff we use elsewhere only borrows the
                // `RegisterBlock`, so we are unable to move a field out.
                sfsi2c0: unsafe { core::ptr::read(pac::SCU::ptr()).sfsi2c0 },
                $([<port_ $x>]: [<Port $X>]{ _marker: PhantomData },)+
            }
        }
    }}
}

ports! {
    0 (U0): [
        0 (U0): [
            gpio[0][0]: (function_0_default, Uninit),
        ],
        1 (U1): [
            gpio[0][1]: (function_0_default, Uninit),
        ],
    ],
    1 (U1): [
        0 (U0): [
            gpio[0][4]: (function_0_default, Uninit),
        ],
        1 (U1): [
            gpio[0][8]: (function_0_default, Uninit),
        ],
        2 (U2): [
            gpio[0][9]: (function_0_default, Uninit),
        ],
        3 (U3): [
            gpio[0][10]: (function_0_default, Uninit),
        ],
        4 (U4): [
            gpio[0][11]: (function_0_default, Uninit),
        ],
        5 (U5): [
            gpio[1][8]: (function_0_default, Uninit),
        ],
        6 (U6): [
            gpio[1][9]: (function_0_default, Uninit),
        ],
        7 (U7): [
            gpio[1][0]: (function_0_default, Uninit),
        ],
        8 (U8): [
            gpio[1][1]: (function_0_default, Uninit),
        ],
        9 (U9): [
            gpio[1][2]: (function_0_default, Uninit),
        ],
        10 (U10): [
            gpio[1][3]: (function_0_default, Uninit),
        ],
        11 (U11): [
            gpio[1][4]: (function_0_default, Uninit),
        ],
        12 (U12): [
            gpio[1][5]: (function_0_default, Uninit),
        ],
        13 (U13): [
            gpio[1][6]: (function_0_default, Uninit),
        ],
        14 (U14): [
            gpio[1][7]: (function_0_default, Uninit),
        ],
        15 (U15): [
            gpio[0][2]: (function_0_default, Uninit),
        ],
        16 (U16): [
            gpio[0][3]: (function_0_default, Uninit),
        ],
        17 (U17): [
            gpio[0][12]: (function_0_default, Uninit),
        ],
        18 (U18): [
            gpio[0][13]: (function_0_default, Uninit),
        ],
        // mind the gap
        20 (U20): [
            gpio[0][15]: (function_0_default, Uninit),
        ],
    ],
    2 (U2): [
        0 (U0): [
            gpio[5][0]: (function_4, Uninit),
        ],
        1 (U1): [
            gpio[5][1]: (function_4, Uninit),
        ],
        2 (U2): [
            gpio[5][2]: (function_4, Uninit),
        ],
        3 (U3): [
            gpio[5][3]: (function_4, Uninit),
        ],
        4 (U4): [
            gpio[5][4]: (function_4, Uninit),
        ],
        5 (U5): [
            gpio[5][5]: (function_4, Uninit),
        ],
        6 (U6): [
            gpio[5][6]: (function_4, Uninit),
        ],
        7 (U7): [
            gpio[0][7]: (function_0_default, Uninit),
        ],
        8 (U8): [
            gpio[5][7]: (function_4, Uninit),
        ],
        9 (U9): [
            gpio[1][10]: (function_0_default, Uninit),
        ],
        11 (U11): [
            gpio[1][11]: (function_0_default, Uninit),
        ],
        12 (U12): [
            gpio[1][12]: (function_0_default, Uninit),
        ],
        13 (U13): [
            gpio[1][13]: (function_0_default, Uninit),
        ],
    ],
    3 (U3): [
        1 (U1): [
            gpio[5][8]: (function_4, Uninit),
        ],
        2 (U2): [
            gpio[5][9]: (function_4, Uninit),
        ],
        4 (U4): [
            gpio[1][14]: (function_0_default, Uninit),
        ],
        5 (U5): [
            gpio[1][15]: (function_0_default, Uninit),
        ],
        6 (U6): [
            gpio[0][6]: (function_0_default, Uninit),
        ],
        7 (U7): [
            gpio[5][10]: (function_4, Uninit),
        ],
        8 (U8): [
            gpio[5][11]: (function_4, Uninit),
        ],
    ],
    4 (U4): [
        0 (U0): [
            gpio[2][0]: (function_0_default, Uninit),
        ],
        1 (U1): [
            gpio[2][1]: (function_0_default, Uninit),
        ],
        2 (U2): [
            gpio[2][2]: (function_0_default, Uninit),
        ],
        3 (U3): [
            gpio[2][3]: (function_0_default, Uninit),
        ],
        4 (U4): [
            gpio[2][4]: (function_0_default, Uninit),
        ],
        5 (U5): [
            gpio[2][5]: (function_0_default, Uninit),
        ],
        6 (U6): [
            gpio[2][6]: (function_0_default, Uninit),
        ],
        8 (U8): [
            gpio[5][12]: (function_4, Uninit),
        ],
        9 (U9): [
            gpio[5][13]: (function_4, Uninit),
        ],
        10 (U10): [
            gpio[5][14]: (function_4, Uninit),
        ],
    ],
    5 (U5): [
        0 (U0): [
            gpio[2][9]: (function_0_default, Uninit),
        ],
        1 (U1): [
            gpio[2][10]: (function_0_default, Uninit),
        ],
        2 (U2): [
            gpio[2][11]: (function_0_default, Uninit),
        ],
        3 (U3): [
            gpio[2][12]: (function_0_default, Uninit),
        ],
        4 (U4): [
            gpio[2][13]: (function_0_default, Uninit),
        ],
        5 (U5): [
            gpio[2][14]: (function_0_default, Uninit),
        ],
        6 (U6): [
            gpio[2][15]: (function_0_default, Uninit),
        ],
        7 (U7): [
            gpio[2][7]: (function_0_default, Uninit),
        ],
    ],
    6 (U6): [
        1 (U1): [
            gpio[3][0]: (function_0_default, Uninit),
        ],
        2 (U2): [
            gpio[3][1]: (function_0_default, Uninit),
        ],
        3 (U3): [
            gpio[3][2]: (function_0_default, Uninit),
        ],
        4 (U4): [
            gpio[3][3]: (function_0_default, Uninit),
        ],
        5 (U5): [
            gpio[3][4]: (function_0_default, Uninit),
        ],
        6 (U6): [
            gpio[0][5]: (function_0_default, Uninit),
        ],
        7 (U7): [
            gpio[5][15]: (function_4, Uninit),
        ],
        8 (U8): [
            gpio[5][16]: (function_4, Uninit),
        ],
        9 (U9): [
            gpio[3][5]: (function_0_default, Uninit),
        ],
        10 (U10): [
            gpio[3][6]: (function_0_default, Uninit),
        ],
        11 (U11): [
            gpio[3][7]: (function_0_default, Uninit),
        ],
        12 (U12): [
            gpio[2][8]: (function_0_default, Uninit),
        ],
    ],
    7 (U7): [
        0 (U0): [
            gpio[3][8]: (function_0_default, Uninit),
        ],
        1 (U1): [
            gpio[3][9]: (function_0_default, Uninit),
        ],
        2 (U2): [
            gpio[3][10]: (function_0_default, Uninit),
        ],
        3 (U3): [
            gpio[3][11]: (function_0_default, Uninit),
        ],
        4 (U4): [
            gpio[3][12]: (function_0_default, Uninit),
        ],
        5 (U5): [
            gpio[3][13]: (function_0_default, Uninit),
        ],
        6 (U6): [
            gpio[3][14]: (function_0_default, Uninit),
        ],
        7 (U7): [
            gpio[3][15]: (function_0_default, Uninit),
        ],
    ],
    8 (U8): [
        0 (U0): [
            gpio[4][0]: (function_0_default, Uninit),
        ],
        1 (U1): [
            gpio[4][1]: (function_0_default, Uninit),
        ],
        2 (U2): [
            gpio[4][2]: (function_0_default, Uninit),
        ],
        3 (U3): [
            gpio[4][3]: (function_0_default, Uninit),
        ],
        4 (U4): [
            gpio[4][4]: (function_0_default, Uninit),
        ],
        5 (U5): [
            gpio[4][5]: (function_0_default, Uninit),
        ],
        6 (U6): [
            gpio[4][6]: (function_0_default, Uninit),
        ],
        7 (U7): [
            gpio[4][7]: (function_0_default, Uninit),
        ],
    ],
    9 (U9): [
        0 (U0): [
            gpio[4][12]: (function_0_default, Uninit),
        ],
        1 (U1): [
            gpio[4][13]: (function_0_default, Uninit),
        ],
        2 (U2): [
            gpio[4][14]: (function_0_default, Uninit),
        ],
        3 (U3): [
            gpio[4][15]: (function_0_default, Uninit),
        ],
        4 (U4): [
            gpio[5][17]: (function_4, Uninit),
        ],
        5 (U5): [
            gpio[5][18]: (function_4, Uninit),
        ],
        6 (U6): [
            gpio[4][11]: (function_0_default, Uninit),
        ],
    ],
    a (A): [
        1 (U1): [
            gpio[4][8]: (function_0_default, Uninit),
        ],
        2 (U2): [
            gpio[4][9]: (function_0_default, Uninit),
        ],
        3 (U3): [
            gpio[4][10]: (function_0_default, Uninit),
        ],
        4 (U4): [
            gpio[5][19]: (function_4, Uninit),
        ],
    ],
    b (B): [
        0 (U0): [
            gpio[5][20]: (function_4, Uninit),
        ],
        1 (U1): [
            gpio[5][21]: (function_4, Uninit),
        ],
        2 (U2): [
            gpio[5][22]: (function_4, Uninit),
        ],
        3 (U3): [
            gpio[5][23]: (function_4, Uninit),
        ],
        4 (U4): [
            gpio[5][24]: (function_4, Uninit),
        ],
        5 (U5): [
            gpio[5][25]: (function_4, Uninit),
        ],
        6 (U6): [
            gpio[5][26]: (function_4, Uninit),
        ],
    ],
    c (C): [
        1 (U1): [
            gpio[6][0]: (function_4, Uninit),
        ],
        2 (U2): [
            gpio[6][1]: (function_4, Uninit),
        ],
        3 (U3): [
            gpio[6][2]: (function_4, Uninit),
        ],
        4 (U4): [
            gpio[6][3]: (function_4, Uninit),
        ],
        5 (U5): [
            gpio[6][4]: (function_4, Uninit),
        ],
        6 (U6): [
            gpio[6][5]: (function_4, Uninit),
        ],
        7 (U7): [
            gpio[6][6]: (function_4, Uninit),
        ],
        8 (U8): [
            gpio[6][7]: (function_4, Uninit),
        ],
        9 (U9): [
            gpio[6][8]: (function_4, Uninit),
        ],
        10 (U10): [
            gpio[6][9]: (function_4, Uninit),
        ],
        11 (U11): [
            gpio[6][10]: (function_4, Uninit),
        ],
        12 (U12): [
            gpio[6][11]: (function_4, Uninit),
        ],
        13 (U13): [
            gpio[6][12]: (function_4, Uninit),
        ],
        14 (U14): [
            gpio[6][13]: (function_4, Uninit),
        ],
    ],
    d (D): [
        0 (U0): [
            gpio[6][14]: (function_4, Uninit),
        ],
        1 (U1): [
            gpio[6][15]: (function_4, Uninit),
        ],
        2 (U2): [
            gpio[6][16]: (function_4, Uninit),
        ],
        3 (U3): [
            gpio[6][17]: (function_4, Uninit),
        ],
        4 (U4): [
            gpio[6][18]: (function_4, Uninit),
        ],
        5 (U5): [
            gpio[6][19]: (function_4, Uninit),
        ],
        6 (U6): [
            gpio[6][20]: (function_4, Uninit),
        ],
        7 (U7): [
            gpio[6][21]: (function_4, Uninit),
        ],
        8 (U8): [
            gpio[6][22]: (function_4, Uninit),
        ],
        9 (U9): [
            gpio[6][23]: (function_4, Uninit),
        ],
        10 (U10): [
            gpio[6][24]: (function_4, Uninit),
        ],
        11 (U11): [
            gpio[6][25]: (function_4, Uninit),
        ],
        12 (U12): [
            gpio[6][26]: (function_4, Uninit),
        ],
        13 (U13): [
            gpio[6][27]: (function_4, Uninit),
        ],
        14 (U14): [
            gpio[6][28]: (function_4, Uninit),
        ],
        15 (U15): [
            gpio[6][29]: (function_4, Uninit),
        ],
        16 (U16): [
            gpio[6][30]: (function_4, Uninit),
        ],
    ],
    e (E): [
        0 (U0): [
            gpio[7][0]: (function_4, Uninit),
        ],
        1 (U1): [
            gpio[7][1]: (function_4, Uninit),
        ],
        2 (U2): [
            gpio[7][2]: (function_4, Uninit),
        ],
        3 (U3): [
            gpio[7][3]: (function_4, Uninit),
        ],
        4 (U4): [
            gpio[7][4]: (function_4, Uninit),
        ],
        5 (U5): [
            gpio[7][5]: (function_4, Uninit),
        ],
        6 (U6): [
            gpio[7][6]: (function_4, Uninit),
        ],
        7 (U7): [
            gpio[7][7]: (function_4, Uninit),
        ],
        8 (U8): [
            gpio[7][8]: (function_4, Uninit),
        ],
        9 (U9): [
            gpio[7][9]: (function_4, Uninit),
        ],
        10 (U10): [
            gpio[7][10]: (function_4, Uninit),
        ],
        11 (U11): [
            gpio[7][11]: (function_4, Uninit),
        ],
        12 (U12): [
            gpio[7][12]: (function_4, Uninit),
        ],
        13 (U13): [
            gpio[7][13]: (function_4, Uninit),
        ],
        14 (U14): [
            gpio[7][14]: (function_4, Uninit),
        ],
        15 (U15): [
            gpio[7][15]: (function_4, Uninit),
        ],
    ],
    f (F): [
        1 (U1): [
            gpio[7][16]: (function_4, Uninit),
        ],
        2 (U2): [
            gpio[7][17]: (function_4, Uninit),
        ],
        3 (U3): [
            gpio[7][18]: (function_4, Uninit),
        ],
        5 (U5): [
            gpio[7][19]: (function_4, Uninit),
        ],
        6 (U6): [
            gpio[7][20]: (function_4, Uninit),
        ],
        7 (U7): [
            gpio[7][21]: (function_4, Uninit),
        ],
        8 (U8): [
            gpio[7][22]: (function_4, Uninit),
        ],
        9 (U9): [
            gpio[7][23]: (function_4, Uninit),
        ],
        10 (U10): [
            gpio[7][24]: (function_4, Uninit),
        ],
        11 (U11): [
            gpio[7][25]: (function_4, Uninit),
        ],
    ],
}
