#![allow(dead_code)]

pub use crate::pac::scu::sfsusb::USB_ESEAW;

#[macro_export]
macro_rules! scu_normal_pin {
    ($scu:expr, $pin:ident, $mode:expr, $pulldown:expr, $pullup:expr, $slew:expr, $buffered:expr, $filter:expr) => {
        $scu.$pin.modify(|_, w| {
            w.mode()
                .bits($mode)
                .epd()
                .bit($pulldown)
                .epun()
                .bit($pullup)
                .ehs()
                .bit($slew)
                .ezi()
                .bit($buffered)
                .zif()
                .bit($filter)
        });
    };
}
// #[macro_export]
// macro_rules! scu_normal_pin {
//     ($scu:expr, $pin:expr, $idx:expr, $mode:expr, $pulldown:expr, $pullup:expr, $slew:expr, $buffered:expr, $filter:expr) => {
//         $scu.$pin[$idx].modify(|_, w| {
//             w.mode()
//                 .variant($mode)
//                 .epd()
//                 .variant($pulldown)
//                 .epun()
//                 .variant($pullup)
//                 .ehs()
//                 .variant($slew)
//                 .ezi()
//                 .variant($buffered)
//                 .zif()
//                 .variant($filter)
//         });
//     };
// }

#[macro_export]
macro_rules! scu_hdrive_pin {
    ($scu:expr, $pin:expr, $mode:expr, $pulldown:expr, $pullup:expr, $slew:expr, $buffered:expr, $filter:expr, $strength:expr) => {
        $scu.$pin.write(|w| {
            w.mode()
                .variant($mode)
                .epd()
                .variant($pulldown)
                .epun()
                .variant($pullup)
                .ehs()
                .variant($slew)
                .ezi()
                .variant($buffered)
                .zif()
                .variant($filter)
                .ehd()
                .variant($strength)
        });
    };
}

#[macro_export]
macro_rules! scu_usb_pin {
    ($scu:expr, $pin:expr, $data:expr, $pulldown:expr, $power:expr, $vbus:expr) => {
        $scu.$pin.write(|w| {
            w.usb_aim()
                .variant($data)
                .usb_esea()
                .variant(SINGLE_INPUT) // 0 = reserved, do not use
                .usb_epd()
                .variant($pulldown)
                .usb_epwr()
                .variant($power)
                .usb_vbus()
                .variant($vbus)
        });
    };
}

#[macro_export]
macro_rules! scu_i2c_pin {
    ($scu:ident, $pin:expr, $filterconstscl:expr, $i2cmodescl:expr, $receiverenscl:expr, $filterenscl:expr, $filterconstsda:expr, $i2cmodesda:expr, $receiverensda:expr, $filterensda:expr) => {
        $scu.$pin.write(|w| {
            w.scl_efp()
                .variant($filterconstscl)
                .scl_ehd()
                .variant($i2cmodescl)
                .scl_ezi()
                .variant($receiverenscl)
                .scl_zif()
                .variant($filterenscl)
                .sda_efp()
                .variant($filterconstsda)
                .sda_ehd()
                .variant($i2cmodesda)
                .sda_ezi()
                .variant($receiverensda)
                .sda_zif()
                .variant($filterensda)
        });
    };
}

/// p. 295
fn i2c0(
    scu: &crate::pac::SCU,
    scl_efp: crate::pac::scu::sfsi2c0::SCL_EFPW,
    scl_ehd: crate::pac::scu::sfsi2c0::SCL_EHDW,
    scl_ezi: crate::pac::scu::sfsi2c0::SCL_EZIW,
    scl_zif: crate::pac::scu::sfsi2c0::SCL_ZIFW,
    sda_efp: crate::pac::scu::sfsi2c0::SDA_EFPW,
    sda_ehd: crate::pac::scu::sfsi2c0::SDA_EHDW,
    sda_ezi: crate::pac::scu::sfsi2c0::SDA_EZIW,
    sda_zif: crate::pac::scu::sfsi2c0::SDA_ZIFW,
) -> () {
    scu.sfsi2c0.write(|w| w.scl_efp().variant(scl_efp));
    scu.sfsi2c0.write(|w| w.scl_ehd().variant(scl_ehd));
    scu.sfsi2c0.write(|w| w.scl_ezi().variant(scl_ezi));
    scu.sfsi2c0.write(|w| w.scl_zif().variant(scl_zif));
    scu.sfsi2c0.write(|w| w.sda_efp().variant(sda_efp));
    scu.sfsi2c0.write(|w| w.sda_ehd().variant(sda_ehd));
    scu.sfsi2c0.write(|w| w.sda_ezi().variant(sda_ezi));
    scu.sfsi2c0.write(|w| w.sda_zif().variant(sda_zif));
}

pub fn uart1(uart1: &crate::pac::UART1) {}
