pub use crate::pac::sct::ev_ctrl::{COMBMODE_A, HEVENT_A, IOCOND_A, OUTSEL_A, STATELD_A};

/// Make event set a bit
pub fn set_event_set(out_set: &crate::pac::sct::OUT_SET, output: u8) {
    assert!(output < 16);
    unsafe {
        out_set.modify(|r, w| w.bits(r.bits() | 1 << output));
    }
}

/// Make event clear a bit
pub fn set_event_clear(out_clr: &crate::pac::sct::OUT_CLR, output: u8) {
    assert!(output < 16);
    unsafe {
        out_clr.modify(|r, w| w.bits(r.bits() | 1 << output));
    }
}

pub fn config_match_event(
    evctrl: &crate::pac::sct::EV_CTRL,
    hevent: HEVENT_A,
    combmode: COMBMODE_A,
    mr: u8,
) {
    unsafe {
        evctrl.modify(|_, w| {
            w.matchsel()
                .bits(mr)
                .combmode()
                .variant(combmode)
                .hevent()
                .variant(hevent)
        });
    }
}

pub fn config_event(
    evctrl: &crate::pac::sct::EV_CTRL,
    mr: u8,
    mode: HEVENT_A,
    outsel: OUTSEL_A,
    iosel: u8,
    iocond: IOCOND_A,
    combmode: COMBMODE_A,
    stateld: STATELD_A,
    statev: u8,
) {
    assert!(mr & 0xf0 == 0);
    assert!(statev & 0xe0 == 0);
    unsafe {
        evctrl.modify(|_, w| {
            w.matchsel()
                .bits(mr)
                .hevent()
                .variant(mode)
                .outsel()
                .variant(outsel)
                .iosel()
                .bits(iosel)
                .iocond()
                .variant(iocond)
                .combmode()
                .variant(combmode)
                .stateld()
                .variant(stateld)
                .statev()
                .bits(statev)
        });
    }
}
