// Configure the SCT, refer to 28.7.10.1 in UM10503
#![allow(dead_code)]
use crate::pac;
pub mod config;
pub mod ctrl;
pub mod event;
pub mod util;

pub fn enable_interrupt(sct: &pac::SCT, event_num: u8) {
    assert!(event_num < 16);
    unsafe {
        sct.even.modify(|r, w| w.bits(r.bits() | 1 << event_num));
    }
}

/// Use mask 0x1 if you're not using states
pub fn set_state_mask(evstate: &pac::sct::EV_STATE, mask: u32) {
    unsafe {
        evstate.write(|w| w.bits(mask));
    }
}

/// Set or clear conflict management enable
pub fn set_conen(sct: &pac::SCT, value: u16) {
    unsafe {
        sct.conen.write(|w| w.bits(value as u32));
    }
}

/// Set or clear conflict management flags
pub fn set_conflags(sct: &pac::SCT, value: u16) {
    unsafe {
        sct.conflag.write(|w| w.bits(value as u32));
    }
}

/// Get low and high error bits
pub fn get_conerror(sct: &pac::SCT) -> u8 {
    return (sct.conflag.read().bits() >> 30) as u8;
}
